@@ -39,12 +39,15 @@ struct _EditorSidebarItem
 
   GFile      *file;
   EditorPage *page;
-  gchar      *search_text;
   gchar      *draft_id;
   gchar      *title;
   gchar      *subtitle;
   gint64      age;
 
+  GPtrArray  *keywords;
+
+  guint       score;
+
   guint       is_modified_set : 1;
   guint       is_modified : 1;
 };
@@ -57,6 +60,7 @@ enum {
   PROP_FILE,
   PROP_IS_MODIFIED,
   PROP_PAGE,
+  PROP_SCORE,
   PROP_SUBTITLE,
   PROP_TITLE,
   N_PROPS
@@ -211,7 +215,7 @@ editor_sidebar_item_notify_title_cb (EditorSidebarItem *self,
   g_assert (EDITOR_IS_SIDEBAR_ITEM (self));
   g_assert (EDITOR_IS_PAGE (page));
 
-  g_clear_pointer (&self->search_text, g_free);
+  g_clear_pointer (&self->keywords, g_ptr_array_unref);
 
   g_clear_pointer (&self->title, g_free);
   self->title = editor_page_dup_title (page);
@@ -227,7 +231,8 @@ editor_sidebar_item_notify_subtitle_cb (EditorSidebarItem *self,
   g_assert (EDITOR_IS_SIDEBAR_ITEM (self));
   g_assert (EDITOR_IS_PAGE (page));
 
-  g_clear_pointer (&self->search_text, g_free);
+  g_clear_pointer (&self->keywords, g_ptr_array_unref);
+
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SUBTITLE]);
 }
 
@@ -280,7 +285,7 @@ editor_sidebar_item_finalize (GObject *object)
 
   g_clear_object (&self->file);
   g_clear_object (&self->page);
-  g_clear_pointer (&self->search_text, g_free);
+  g_clear_pointer (&self->keywords, g_ptr_array_unref);
   g_clear_pointer (&self->draft_id, g_free);
   g_clear_pointer (&self->subtitle, g_free);
 
@@ -321,6 +326,10 @@ editor_sidebar_item_get_property (GObject    *object,
       g_value_set_boolean (value, _editor_sidebar_item_get_is_modified (self));
       break;
 
+    case PROP_SCORE:
+      g_value_set_uint (value, self->score);
+      break;
+
     case PROP_TITLE:
       if (self->title != NULL)
         g_value_set_string (value, self->title);
@@ -415,6 +424,11 @@ editor_sidebar_item_class_init (EditorSidebarItemClass *klass)
                           FALSE,
                           (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  properties [PROP_SCORE] =
+    g_param_spec_uint ("score", NULL, NULL,
+                       0, G_MAXUINT, 0,
+                       (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   properties [PROP_TITLE] =
     g_param_spec_string ("title",
                          "Title",
@@ -544,21 +558,46 @@ gboolean
 _editor_sidebar_item_matches (EditorSidebarItem *self,
                               const char        *search)
 {
-  guint prio;
+  gboolean matches = FALSE;
 
   if (search == NULL)
     return TRUE;
 
-  if G_UNLIKELY (self->search_text == NULL)
+  if G_UNLIKELY (self->keywords == NULL)
     {
       g_autofree gchar *title = _editor_sidebar_item_dup_title (self);
-      g_autofree gchar *title_fold = g_utf8_casefold (title, -1);
-      g_autofree gchar *subtitle_fold = g_utf8_casefold (self->subtitle, -1);
 
-      self->search_text = g_strdup_printf ("%s %s", title_fold, subtitle_fold);
+      self->keywords = g_ptr_array_new_with_free_func (g_free);
+
+      g_ptr_array_add (self->keywords, g_utf8_casefold (title, -1));
+      g_ptr_array_add (self->keywords, g_utf8_casefold (self->subtitle, -1));
     }
 
-  return gtk_source_completion_fuzzy_match (self->search_text, search, &prio);
+  self->score = 0;
+
+  for (guint i = 0; i < self->keywords->len; i++)
+    {
+      const char *keyword = g_ptr_array_index (self->keywords, i);
+      guint score = 0;
+
+      if (g_str_has_prefix (keyword, search))
+        {
+          matches = TRUE;
+          self->score = 0;
+          break;
+        }
+      else if (gtk_source_completion_fuzzy_match (keyword, search, &score))
+        {
+          if (score > self->score)
+            self->score = score;
+          matches = TRUE;
+        }
+    }
+
+  if (matches)
+    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SCORE]);
+
+  return matches;
 }
 
 void
